<?php
namespace platy\etsy\rest\etsy\categories;
use platy\etsy\EtsySyncer;
use platy\etsy\EtsySyncerException;

class EtsyCategoriesApi extends \WP_REST_Controller {
    /**
     *
     * @var int
     */
    private $current_shop;

    /**
     * @var EtsySyncer
     */
    private $syncer;
    public function __construct() {
        $this->namespace     = 'platy-syncer/v1';
        $this->resource_name = 'etsy-categories';
        $this->syncer = new EtsySyncer();
    }

    public function register_routes() {

        \register_rest_route( $this->namespace, '/' . $this->resource_name, array(
            array(
                'methods'   => 'GET',
                'callback'  => function($request){ 
                    try{
                        $cats = $this->syncer->get_etsy_taxonomy_tree();
                    }catch(EtsySyncerException $e){
                        return new \WP_Error( 'platy_syncer_error', $e->getMessage(), array( 'status' => 400 ) );
                    }
                    $ret = [];
                    $ret = $this->clean_results($cats, $ret);
                    return $ret;
                },
                'permission_callback' => array( $this, 'permissions_check' ),
            )
        ) );

       
        
    }

    protected function clean_results($cats, &$ret){
        foreach($cats as $cat){
            $cleaned = ['id' => $cat['id'], "name" => EtsySyncer::clean_name($cat['name'])];
            if(!empty($cat['children'])){
                $children = [];
                $cleaned['children'] = $this->clean_results($cat['children'], $children);
            }
            $ret[] = $cleaned;
        }
        return $ret;
    }


    public function permissions_check( $request ) {
        if ( ! current_user_can( 'administrator' ) && ! current_user_can( 'shop_manager' ) ) {
            return new \WP_Error( 'rest_forbidden', esc_html__( 'You do not have sufficient privileges' ), array( 'status' => 403 ) );
        }
        return true;
    }

   
  }