<?php

namespace platy\etsy\orders;
use platy\etsy\EtsyDataService;
use platy\etsy\EtsySyncerException;

class EtsyOrderItem extends \WC_Order_Item_Product{

    private $properties;
    private $transaction_id;
    public function __construct($item_id = 0, $transaction_id = 0 ) {
		parent::__construct( $item_id );
        $this->properties = [];
        if(!empty($transaction_id)){
            $this->transaction_id = $transaction_id;
        }else{

            if(!empty($item_id)){
                $this->transaction_id = wc_get_order_item_meta($item_id, "_etsy_transaction_id", true);
            }
        }
    }


    public function set_etsy_property($prop_name, $prop_value){
        $this->properties[$prop_name] = $prop_value;
    }

    public function save(){
        $id = parent::save();
        foreach($this->properties as $prop_name => $prop_value){
            wc_update_order_item_meta($id, $prop_name, $prop_value);
        }
        wc_update_order_item_meta($id, "_is_platy_etsy_order_item", "true");
        wc_update_order_item_meta($id, "_etsy_transaction_id", $this->transaction_id);

    }

    public function get_transaction_id(){
        return $this->transaction_id;
    }

    public function get_thumbnail(){
        return wc_get_order_item_meta($this->get_id(), "_etsy_image_src_75x75", true);
    }

    public function get_etsy_link(){
        $listing_id = wc_get_order_item_meta($this->get_id(), "_etsy_listing_id", true);
        return "https://etsy.com/listing/$listing_id";
    }

    private function get_woo_id() {
        $item_id = $this->get_id();
        $etsy_product_id = wc_get_order_item_meta($item_id, "_etsy_product_id", true);
        $etsy_shop_id = wc_get_order_item_meta($item_id, "_etsy_shop_id", true);
        $etsy_listing_id = wc_get_order_item_meta($item_id, "_etsy_listing_id", true);
        $data_service = EtsyDataService::get_instance();
        try {
            return $data_service->get_log_product_id($etsy_listing_id, $etsy_shop_id, 'product');
        }catch(EtsySyncerException $e) {

        }

        try {
            return $data_service->get_log_product_id($etsy_product_id, $etsy_shop_id, 'variation');
        }catch(EtsySyncerException $e) {

        }

        $etsy_sku = wc_get_order_item_meta($item_id, "_etsy_sku", true);
        return wc_get_product_id_by_sku($etsy_sku);
    }

    public function get_woo_link() {
        $woo_id = $this->get_woo_id();
        if(empty($woo_id)) {
            return null;
        }

        $parent_id = wp_get_post_parent_id( $woo_id );
        $woo_id = empty($parent_id) ? $woo_id : $parent_id;
        return get_edit_post_link( $woo_id );
    }

    public function get_item_variation_props(){
        $metadata = $this->get_formatted_meta_data('', true);
        // \error_log(\json_encode($metadata));
        $ret = [];
        foreach($metadata as $meta){
            if(\substr($meta->key, 0, 6) != "_prop_"){
                continue;
            }
            $prop_key = \substr($meta->key, 6);
            $ret[$prop_key] = $meta->value;
            
        }

        return $ret;
    }
}